package com.mtcarpenter.mall.mapper;

import com.mtcarpenter.mall.model.UmsCompanyShop;
import com.mtcarpenter.mall.model.UmsCompanyShopExample;
import java.util.List;
import org.apache.ibatis.annotations.Param;

public interface UmsCompanyShopMapper {
    long countByExample(UmsCompanyShopExample example);

    int deleteByExample(UmsCompanyShopExample example);

    int deleteByPrimaryKey(Integer id);

    int insert(UmsCompanyShop record);

    int insertSelective(UmsCompanyShop record);

    List<UmsCompanyShop> selectByExample(UmsCompanyShopExample example);

    UmsCompanyShop selectByPrimaryKey(Integer id);

    int updateByExampleSelective(@Param("record") UmsCompanyShop record, @Param("example") UmsCompanyShopExample example);

    int updateByExample(@Param("record") UmsCompanyShop record, @Param("example") UmsCompanyShopExample example);

    int updateByPrimaryKeySelective(UmsCompanyShop record);

    int updateByPrimaryKey(UmsCompanyShop record);
}